package nxtbluetooth;

import java.io.IOException;
import lejos.nxt.*;
import lejos.robotics.RegulatedMotor;

/**
 * Hlavni trida aplikace.
 *
 * @author Michael
 */
public class Main {

    public static RegulatedMotor mA, mB;
    
    private Comm comm;
    private SonicSensor sSensor;
    private BumpSensor bSensor;
    private Thread sSensorThread;


    /**
     * Zavadi prvni spojeni s PC.
     */
    public Main() {
        comm = new Comm();
        sSensor = new SonicSensor();
        sSensorThread = new Thread(sSensor);
        bSensor = new BumpSensor();
        mA = Motor.A;
        mB = Motor.B;
        comm.setDataInputStream();
        comm.setDataOutputStream();
        
        sSensorThread.start();
    }

    /**
     * Vytvari DataInputStream a odchytava prichozi packety, na ktere nasledne
     * reaguje volanim prislusnych metod.
     * Pri ztrate spojeni zacne NXT znovu naslouchat minule forme spojeni
     * (BT/USB). Pri znovunavazani obnovi i DataInputStream.
     */
    public void listenToPC() {
        int order = 999;

        try {
            while(true) {
                order = comm.getPacketInt();

                if(order == 1000) {
                    // ukoncuje stream
                    motorGo(mA, 0, true);
                    motorGo(mB, 0, true);
                    break;
                }
                else if(order >= 0) {
                    // provadi odpovidajici prikazy
                    reactToPCCommand(order);
                }
            }
        } catch(Exception io) {
            motorGo(mA, 0, true);
            motorGo(mB, 0, true);
            comm.reconnect();
            comm.setDataInputStream();
            comm.setDataOutputStream();
            // do again
            listenToPC();
        }
    }

    /**
     * Spousti prislusne prikazy podle ciselneho vstupu odpovidajiciho prikazu.
     * @param order Ciselny prikaz k provedeni.
     */
    private void reactToPCCommand(int order) throws IOException {
        switch(order) {
            case 0:
                // stop motors
                motorGo(mA, 0, true);
                motorGo(mB, 0, true);
                printToLCD("stop", 3);
                break;
            case 1:
                // up
                if(scanPath(true)) {
                    motorGo(mA, 500, true);
                    motorGo(mB, 500, true);
                    printToLCD("up", 3);
                }
                break;
            case 2:
                // up + right
                if(scanPath(true)) {
                    motorGo(mA, 500, true);
                    motorGo(mB, 250, true);
                    printToLCD("up + right", 3);
                }
                break;
            case 3:
                // right
                motorGo(mA, 500, true);
                motorGo(mB, 500, false);
                printToLCD("right", 3);
                break;
            case 4:
                // down + right
                if(scanPath(false)) {
                    motorGo(mA, 500, false);
                    motorGo(mB, 250, false);
                    printToLCD("down + right", 3);
                }
                break;
            case 5:
                // down
                if(scanPath(false)) {
                    motorGo(mA, 500, false);
                    motorGo(mB, 500, false);
                    printToLCD("down", 3);
                }
                break;
            case 6:
                // down + left
                if(scanPath(false)) {
                    motorGo(mA, 250, false);
                    motorGo(mB, 500, false);
                    printToLCD("down + left", 3);
                }
                break;
            case 7:
                // left
                motorGo(mA, 500, false);
                motorGo(mB, 500, true);
                printToLCD("left", 3);
                break;
            case 8:
                // up + left
                if(scanPath(true)) {
                    motorGo(mA, 250, true);
                    motorGo(mB, 500, true);
                    printToLCD("up + left", 3);
                }
                break;
            case 10:
                // space
                motorGo(mA, 0, true);
                motorGo(mB, 0, true);
                printToLCD("space", 3);
                break;
        }
    }

    /**
     * Rozjede prislusny motor urcitou rychlosti dopredu ci dozadu.
     * @param type Ktery motor (napr. Motor.A).
     * @param speed 0 - 900 (0 = stop, 900 = max).
     * @param forward True pro jizdu dopredu.
     */
    public static void motorGo(RegulatedMotor type, int speed, boolean forward) {
        if(speed == 0) {
            type.stop(true);
        } else {
            type.setSpeed(speed);
            if(forward) {
                type.forward();
            } else {
                type.backward();
            }
        }
    }
    
    /**
     * Oskenuje cestu pres "Sonic Sensor" ci "Touch Sensor". Pri nedostatku
     * mista pro "SS" ci stisknutem "TS" zastavi motory a odesle PC zpravu.
     * @param front True pro pouziti "SS", jinak "TS".
     * @return Zda je cesta volna.
     */
    public boolean scanPath(boolean front) {
        try {
            sSensorThread.sleep(1);
        } catch (InterruptedException ex) {
            // nothing to do
        }
        if(front && sSensor.isPathClear()) {
            return true;
        }
        else if(!front && bSensor.isNotCrashed()) {
            return true;
        }
        else {
            motorGo(mA, 0, true);
            motorGo(mB, 0, true);
            comm.sendPacket("Obstacle");
            
            return false;
        }
    }
    
    /**
     * Vypisuje zpravu na LCD displej.
     * @param text Zprava k vypsani.
     * @param row Radek, na ktery ma vypsat.
     */
    public static void printToLCD(String text, int row) {
        LCD.clear(row);
        LCD.drawString(text, 0, row);
    }
    
    /**
     * Uzavre vsechna spojeni a ukonci aplikaci.
     */
    public void end() {
        try {
            sSensorThread.sleep(1);
            sSensor.requestStop();
        } catch (InterruptedException ex) {
            // nothing to do
        }
        comm.closeAllConnections();
    }

    /**
     * Hlavni metoda aplikace.
     * @param args
     */
    public static void main(String[] args) {
        Main m = new Main();

        m.listenToPC();
        m.end();
    }

}